import secrets
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from pydantic import AnyHttpUrl, field_validator
from pydantic_settings import BaseSettings

# Try multiple possible database locations in order of preference
DB_LOCATIONS = [
    Path("/app/storage/db"),
    Path("/tmp/taskmanager/db"),
    Path.home() / ".taskmanager/db",
    Path.cwd() / "db"
]

# Try to find or create a writable database directory
DB_DIR = None
for location in DB_LOCATIONS:
    try:
        location.mkdir(parents=True, exist_ok=True)
        # Check if directory is writable by creating a test file
        test_file = location / "test_write"
        test_file.touch()
        test_file.unlink()  # Remove test file
        DB_DIR = location
        print(f"Using database directory: {DB_DIR}")
        break
    except Exception as e:
        print(f"Could not use {location} for database: {e}")

# If no location works, fall back to current directory
if DB_DIR is None:
    DB_DIR = Path.cwd()
    print(f"Falling back to current directory for database: {DB_DIR}")

class Settings(BaseSettings):
    PROJECT_NAME: str = "Task Manager API"
    API_V1_STR: str = "/api/v1"
    SECRET_KEY: str = secrets.token_urlsafe(32)
    # 60 minutes * 24 hours * 8 days = 8 days
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 60 * 24 * 8
    
    # CORS Configuration
    BACKEND_CORS_ORIGINS: List[AnyHttpUrl] = []

    @field_validator("BACKEND_CORS_ORIGINS", mode="before")
    def assemble_cors_origins(cls, v: Union[str, List[str]]) -> Union[List[str], str]:
        if isinstance(v, str) and not v.startswith("["):
            return [i.strip() for i in v.split(",")]
        elif isinstance(v, (list, str)):
            return v
        raise ValueError(v)

    # Database configuration
    SQLALCHEMY_DATABASE_URL: str = f"sqlite:///{DB_DIR}/db.sqlite"
    
    model_config = {
        "case_sensitive": True
    }


settings = Settings()